"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const events_1 = __importDefault(require("events"));
class CustomEvents {
    constructor() {
        this.events = new events_1.default();
        this.events.setMaxListeners(0);
        this.pending = {};
    }
    once(event, listener) {
        this.events.once(event, listener);
        return () => this.events.removeListener(event, listener);
    }
    wait(successEvent, rejectEvents, timeoutSeconds = 60 * 2 // 2 minutes
    ) {
        return new Promise((resolve, reject) => {
            if (this.pending[successEvent]) {
                this.pending[successEvent].clear();
                this.pending[successEvent].reject(Error(`Retrying`));
                delete this.pending[successEvent];
            }
            let onSuccess;
            let onFailure;
            let timeout;
            const clear = () => clearTimeout(timeout);
            const removeListeners = () => {
                delete this.pending[successEvent];
                this.events.removeListener(successEvent, onSuccess);
                if (rejectEvents) {
                    if (typeof rejectEvents === "string") {
                        this.events.removeListener(rejectEvents, onFailure);
                    }
                    else if (Array.isArray(rejectEvents)) {
                        for (const event of rejectEvents) {
                            this.events.removeListener(event, onFailure);
                        }
                    }
                }
            };
            const reset = (time = 60 * 2) => {
                clear();
                timeout = setTimeout(() => {
                    removeListeners();
                    reject(Error(`Timeout`));
                }, time * 1000);
            };
            onSuccess = (params) => {
                clear();
                removeListeners();
                resolve(params);
            };
            onFailure = (message) => {
                clear();
                removeListeners();
                reject(Error(message || "no reason"));
            };
            this.pending[successEvent] = { reject, clear, reset };
            reset(timeoutSeconds);
            this.events.once(successEvent, onSuccess);
            if (rejectEvents) {
                if (typeof rejectEvents === "string") {
                    this.events.once(rejectEvents, onFailure);
                }
                else if (Array.isArray(rejectEvents)) {
                    for (const event of rejectEvents) {
                        this.events.once(event, onFailure);
                    }
                }
            }
        });
    }
    resetTimeout(successEvent, timeoutSeconds) {
        if (this.pending[successEvent]) {
            this.pending[successEvent].reset(timeoutSeconds);
            return true;
        }
        return false;
    }
    emit(event, params) {
        this.events.emit(event, params);
    }
    removeAllListeners(reason) {
        this.events.removeAllListeners();
        for (const id in this.pending) {
            try {
                this.pending[id].clear();
                this.pending[id].reject(Error(reason || "no reason"));
            }
            catch (err) { }
        }
        this.pending = {};
    }
}
exports.default = CustomEvents;
//# sourceMappingURL=events.js.map