/// <reference types="node" />
/// <reference types="node" />
/// <reference types="node" />
import EventEmitter from "events";
import Net from "net";
import { Block, Transaction, Header } from "bsv-minimal";
import { NetAddress } from "./messages/address";
import { GetHeadersOptions } from "./messages/headers";
import { VersionOptions } from "./messages/version";
import CustomEvents from "./events";
export interface PeerOptions {
    node: string;
    port?: number;
    ticker: string;
    stream?: boolean;
    validate?: boolean;
    autoReconnect?: boolean;
    autoReconnectWait?: number;
    disableExtmsg?: boolean;
    segwit?: boolean;
    DEBUG_LOG?: boolean;
    magic?: Buffer /** 4 byte Buffer */;
    version?: number;
    user_agent?: string;
    start_height?: number;
    mempoolTxs?: boolean;
}
export type GetBlockReturn = {
    ticker: string;
    blockHash: Buffer;
    header: Header;
    height?: number;
    size: number;
    startDate: number;
};
export default class Peer extends EventEmitter {
    node: string;
    port: number;
    ticker: string;
    magic: Buffer;
    version: number;
    user_agent?: string;
    segwit?: boolean;
    start_height?: number;
    mempoolTxs: boolean;
    stream: boolean;
    validate: boolean;
    autoReconnect: boolean;
    autoReconnectWait: number;
    disableExtmsg: boolean;
    connected: boolean;
    listenTxs?: (txids: Buffer[]) => Promise<Buffer[]> | Buffer[];
    listenBlocks?: (hashes: Buffer[]) => Promise<Buffer[]> | Buffer[];
    extmsg: boolean;
    disconnects: number;
    timeoutConnect: number;
    DEBUG_LOG: boolean;
    emitter: CustomEvents;
    buffers: {
        data: Buffer[];
        needed: number;
        length: number;
        block: Block;
        chunkNum: number;
        downloadingBlock: boolean;
    };
    socket?: Net.Socket | null;
    connectOptions?: VersionOptions;
    promiseConnect?: any;
    constructor({ node, port, ticker, stream, validate, autoReconnect, autoReconnectWait, // 2 seconds
    segwit, disableExtmsg, DEBUG_LOG, magic, version, user_agent, start_height, mempoolTxs, }: PeerOptions);
    sendMessage(command: string, payload: Buffer | null, force?: boolean): void;
    streamBlock(chunk: Buffer): void;
    readMessage(buffer: Buffer): void;
    connect(options?: VersionOptions | undefined): any;
    disconnect(autoReconnect?: boolean): void;
    getHeaders({ from, to, timeoutSeconds, }: {
        from?: GetHeadersOptions["from"];
        to?: GetHeadersOptions["to"];
        timeoutSeconds?: number;
    }): Promise<Header[]>;
    getMempool(): void;
    getBlock(hash: Buffer | string, timeoutSeconds?: number): Promise<GetBlockReturn>;
    getBlocks(blocks: Buffer[]): void;
    broadcastTx(transaction: Transaction, timeoutSeconds?: number): Promise<void>;
    broadcastTxs(transactions: Transaction[], timeoutSeconds?: number): Promise<PromiseSettledResult<void>[]>;
    getTxs(txs: Buffer[]): void;
    getAddr(timeoutSeconds?: number): Promise<{
        ticker: string;
        node: string;
        port: number;
        addrs: NetAddress[];
    }>;
    ping(timeoutSeconds?: number): Promise<number>;
    fetchMempoolTxs(filterTxids: (txids: Buffer[]) => Promise<Buffer[]> | Buffer[]): void;
    fetchNewBlocks(filterBlocks: (hashes: Buffer[]) => Promise<Buffer[]> | Buffer[]): void;
}
//# sourceMappingURL=index.d.ts.map