"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bsv_minimal_1 = require("bsv-minimal");
const { BufferReader, BufferWriter } = bsv_minimal_1.utils;
function getheaders({ from, to, version }) {
    if (!from)
        from = Buffer.alloc(32, 0);
    if (!Array.isArray(from))
        from = [from];
    if (!to)
        to = Buffer.alloc(32, 0);
    if (!Buffer.isBuffer(to))
        to = Buffer.from(to, "hex");
    const bw = new BufferWriter();
    bw.writeUInt32LE(version);
    bw.writeVarintNum(from.length);
    for (const hash of from) {
        bw.writeReverse(hash);
    }
    bw.writeReverse(to);
    return bw.toBuffer();
}
function parseHeaders(payload) {
    const br = new BufferReader(payload);
    const count = br.readVarintNum();
    const headers = [];
    const txs = [];
    for (let i = 0; i < count; i++) {
        const header = bsv_minimal_1.Header.fromBufferReader(br);
        const txCount = br.readVarintNum(); // Always 0
        headers.push(header);
        txs.push(txCount);
    }
    if (!br.eof())
        throw new Error(`Invalid payload`);
    return { headers, txs };
}
const Headers = {
    getheaders,
    parseHeaders,
};
exports.default = Headers;
//# sourceMappingURL=headers.js.map