"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const bsv_minimal_1 = require("bsv-minimal");
const { BufferReader, BufferWriter } = bsv_minimal_1.utils;
function read(buffer) {
    const br = new BufferReader(buffer);
    const count = br.readVarintNum();
    const errors = [];
    const txs = [];
    const blocks = [];
    const filtered_blocks = [];
    const compact_blocks = [];
    const other = [];
    for (let i = 0; i < count; i++) {
        let type = br.readUInt32LE();
        const hash = br.readReverse(32);
        if (type === 0) {
            errors.push(hash);
        }
        else if (type === 1) {
            txs.push(hash);
        }
        else if (type === 2) {
            blocks.push(hash);
        }
        else if (type === 3) {
            filtered_blocks.push(hash);
        }
        else if (type === 4) {
            compact_blocks.push(hash);
        }
        else {
            other.push([hash, type]);
        }
    }
    if (!br.eof())
        throw new Error(`Invalid payload`);
    return { txs, blocks, errors, filtered_blocks, compact_blocks, other };
}
function write({ txs = [], blocks = [], errors = [], filtered_blocks = [], compact_blocks = [], other = [], }) {
    const bw = new BufferWriter();
    bw.writeVarintNum(txs.length +
        blocks.length +
        errors.length +
        filtered_blocks.length +
        compact_blocks.length +
        other.length);
    for (const hash of errors) {
        bw.writeUInt32LE(0);
        bw.writeReverse(hash);
    }
    for (const hash of txs) {
        bw.writeUInt32LE(1);
        bw.writeReverse(hash);
    }
    for (const hash of blocks) {
        bw.writeUInt32LE(2);
        bw.writeReverse(hash);
    }
    for (const hash of filtered_blocks) {
        bw.writeUInt32LE(3);
        bw.writeReverse(hash);
    }
    for (const hash of compact_blocks) {
        bw.writeUInt32LE(4);
        bw.writeReverse(hash);
    }
    for (const [hash, type] of other) {
        bw.writeUInt32LE(type);
        bw.writeReverse(hash);
    }
    return bw.toBuffer();
}
const Inv = {
    read,
    write,
};
exports.default = Inv;
//# sourceMappingURL=inv.js.map